@@ -18,6 +18,7 @@
 
 // Other libraries and framework includes
 // Project includes
+#include "lldb/lldb-private-forward.h"
 #include "lldb/lldb-public.h"
 #include "lldb/Core/ArchSpec.h"
 #include "lldb/Core/ConstString.h"
@@ -25,6 +26,10 @@
 #include "lldb/Interpreter/Options.h"
 #include "lldb/Host/Mutex.h"
 
+// TODO pull NativeDelegate class out of NativeProcessProtocol so we
+// can just forward ref the NativeDelegate rather than include it here.
+#include "../../../source/Host/common/NativeProcessProtocol.h"
+
 namespace lldb_private {
 
     //----------------------------------------------------------------------
@@ -40,7 +45,8 @@ namespace lldb_private {
     ///     @li listing and getting info for existing processes
     ///     @li attaching and possibly debugging the platform's kernel
     //----------------------------------------------------------------------
-    class Platform : public PluginInterface
+    class Platform :
+        public PluginInterface
     {
     public:
 
@@ -173,13 +179,13 @@ namespace lldb_private {
         ///     m_arch - The architecture we are looking for when resolving
         ///              the symbol file.
         ///     m_uuid - The UUID of the executable and symbol file. This
-        ///              can often be used to match up an exectuable with
+        ///              can often be used to match up an executable with
         ///              a symbol file, or resolve an symbol file in a
         ///              symbol file bundle.
         ///
         /// @param[out] sym_file
         ///     The resolved symbol file spec if the returned error
-        ///     indicates succes.
+        ///     indicates success.
         ///
         /// @return
         ///     Returns an error that describes success or failure.
@@ -214,8 +220,7 @@ namespace lldb_private {
         bool
         GetOSKernelDescription (std::string &s);
 
-        // Returns the the hostname if we are connected, else the short plugin
-        // name.
+        // Returns the name of the platform
         ConstString
         GetName ();
 
@@ -269,6 +274,15 @@ namespace lldb_private {
         {
             return ArchSpec(); // Return an invalid architecture
         }
+        
+        virtual ConstString
+        GetRemoteWorkingDirectory()
+        {
+            return m_working_dir;
+        }
+        
+        virtual bool
+        SetRemoteWorkingDirectory(const ConstString &path);
 
         virtual const char *
         GetUserName (uint32_t uid);
@@ -305,9 +319,9 @@ namespace lldb_private {
         ///     An error object.
         //------------------------------------------------------------------
         virtual Error
-        GetFile (const FileSpec &platform_file, 
-                 const UUID *uuid_ptr,
-                 FileSpec &local_file);
+        GetFileWithUUID (const FileSpec &platform_file, 
+                         const UUID *uuid_ptr,
+                         FileSpec &local_file);
 
         //----------------------------------------------------------------------
         // Locate the scripting resource given a module specification.
@@ -340,7 +354,7 @@ namespace lldb_private {
         ///     A zero based architecture index
         ///
         /// @param[out] arch
-        ///     A copy of the archgitecture at index if the return value is
+        ///     A copy of the architecture at index if the return value is
         ///     \b true.
         ///
         /// @return
@@ -384,10 +398,13 @@ namespace lldb_private {
         }
 
         //------------------------------------------------------------------
-        /// Subclasses should NOT need to implement this function as it uses
-        /// the Platform::LaunchProcess() followed by Platform::Attach ()
+        /// Subclasses do not need to implement this function as it uses
+        /// the Platform::LaunchProcess() followed by Platform::Attach ().
+        /// Remote platforms will want to subclass this function in order
+        /// to be able to intercept STDIO and possibly launch a separate
+        /// process that will debug the debuggee.
         //------------------------------------------------------------------
-        lldb::ProcessSP
+        virtual lldb::ProcessSP
         DebugProcess (ProcessLaunchInfo &launch_info,
                       Debugger &debugger,
                       Target *target,       // Can be NULL, if NULL create a new target, else use existing one
@@ -401,7 +418,7 @@ namespace lldb_private {
         /// attempt to attach to the process with the process ID of \a pid.
         /// The platform subclass should return an appropriate ProcessSP 
         /// subclass that is attached to the process, or an empty shared 
-        /// pointer with an appriopriate error.
+        /// pointer with an appropriate error.
         ///
         /// @param[in] pid
         ///     The process ID that we should attempt to attach to.
@@ -410,7 +427,7 @@ namespace lldb_private {
         ///     An appropriate ProcessSP containing a valid shared pointer
         ///     to the default Process subclass for the platform that is 
         ///     attached to the process, or an empty shared pointer with an
-        ///     appriopriate error fill into the \a error object.
+        ///     appropriate error fill into the \a error object.
         //------------------------------------------------------------------
         virtual lldb::ProcessSP
         Attach (ProcessAttachInfo &attach_info,
@@ -542,6 +559,12 @@ namespace lldb_private {
         {
             m_sdk_build = sdk_build;
         }    
+
+        ConstString
+        GetWorkingDirectory ();
+        
+        bool
+        SetWorkingDirectory (const ConstString &path);
         
         // There may be modules that we don't want to find by default for operations like "setting breakpoint by name".
         // The platform will return "true" from this call if the passed in module happens to be one of these.
@@ -552,23 +575,19 @@ namespace lldb_private {
             return false;
         }
         
-        virtual uint32_t
-        MakeDirectory (const std::string &path,
-                       mode_t mode)
-        {
-            return UINT32_MAX;
-        }
-        
-        // this need not be virtual: the core behavior is in
-        // MakeDirectory(std::string,mode_t)
-        uint32_t
-        MakeDirectory (const FileSpec &spec,
-                       mode_t mode);
+        virtual Error
+        MakeDirectory (const char *path, uint32_t permissions);
         
+        virtual Error
+        GetFilePermissions (const char *path, uint32_t &file_permissions);
+
+        virtual Error
+        SetFilePermissions (const char *path, uint32_t file_permissions);
+
         virtual lldb::user_id_t
         OpenFile (const FileSpec& file_spec,
                   uint32_t flags,
-                  mode_t mode,
+                  uint32_t mode,
                   Error &error)
         {
             return UINT64_MAX;
@@ -609,29 +628,55 @@ namespace lldb_private {
             return -1;
         }
         
+        virtual Error
+        GetFile (const FileSpec& source,
+                 const FileSpec& destination);
+        
         virtual Error
         PutFile (const FileSpec& source,
                  const FileSpec& destination,
                  uint32_t uid = UINT32_MAX,
                  uint32_t gid = UINT32_MAX);
-                
+
+        virtual Error
+        CreateSymlink (const char *src, // The name of the link is in src
+                       const char *dst);// The symlink points to dst
+
+        //----------------------------------------------------------------------
+        /// Install a file or directory to the remote system.
+        ///
+        /// Install is similar to Platform::PutFile(), but it differs in that if
+        /// an application/framework/shared library is installed on a remote
+        /// platform and the remote platform requires something to be done to
+        /// register the application/framework/shared library, then this extra
+        /// registration can be done.
+        ///
+        /// @param[in] src
+        ///     The source file/directory to install on the remote system.
+        ///
+        /// @param[in] dst
+        ///     The destination file/directory where \a src will be installed.
+        ///     If \a dst has no filename specified, then its filename will
+        ///     be set from \a src. It \a dst has no directory specified, it
+        ///     will use the platform working directory. If \a dst has a
+        ///     directory specified, but the directory path is relative, the
+        ///     platform working directory will be prepended to the relative
+        ///     directory.
+        ///
+        /// @return
+        ///     An error object that describes anything that went wrong.
+        //----------------------------------------------------------------------
+        virtual Error
+        Install (const FileSpec& src, const FileSpec& dst);
+
         virtual size_t
         GetEnvironment (StringList &environment);
         
-        virtual Error
-        GetFile (const FileSpec& source,
-                 const FileSpec& destination);
-        
         virtual bool
         GetFileExists (const lldb_private::FileSpec& file_spec);
         
-        virtual uint32_t
-        GetFilePermissions (const lldb_private::FileSpec &file_spec,
-                            Error &error)
-        {
-            error.SetErrorStringWithFormat ("Platform::GetFilePermissions() is not supported in the %s platform", GetName().GetCString());
-            return 0;
-        }
+        virtual Error
+        Unlink (const char *path);
 
         virtual bool
         GetSupportsRSync ()
@@ -795,6 +840,88 @@ namespace lldb_private {
             return LLDB_INVALID_QUEUE_ID;
         }
 
+        //------------------------------------------------------------------
+        /// Provide a list of trap handler function names for this platform
+        ///
+        /// The unwinder needs to treat trap handlers specially -- the stack
+        /// frame may not be aligned correctly for a trap handler (the kernel
+        /// often won't perturb the stack pointer, or won't re-align it properly,
+        /// in the process of calling the handler) and the frame above the handler
+        /// needs to be treated by the unwinder's "frame 0" rules instead of its
+        /// "middle of the stack frame" rules.
+        /// 
+        /// In a user process debugging scenario, the list of trap handlers is
+        /// typically just "_sigtramp".
+        ///
+        /// The Platform base class provides the m_trap_handlers ivar but it does
+        /// not populate it.  Subclasses should add the names of the asynchronous
+        /// signal handler routines as needed.  For most Unix platforms, add _sigtramp.
+        ///
+        /// @return
+        ///     A list of symbol names.  The list may be empty.
+        //------------------------------------------------------------------
+        virtual const std::vector<ConstString> &
+        GetTrapHandlerSymbolNames ();
+
+        //------------------------------------------------------------------
+        /// Launch a process for debugging.
+        ///
+        /// This differs from Launch in that it returns a NativeProcessProtocol.
+        /// Currently used by lldb-gdbserver.
+        ///
+        /// @param[in] launch_info
+        ///     Information required to launch the process.
+        ///
+        /// @param[in] native_delegate
+        ///     The delegate that will receive messages regarding the
+        ///     inferior.  Must outlive the NativeProcessProtocol
+        ///     instance.
+        ///
+        /// @param[out] process_sp
+        ///     On successful return from the method, this parameter
+        ///     contains the shared pointer to the
+        ///     NativeProcessProtocol that can be used to manipulate
+        ///     the native process.
+        ///
+        /// @return
+        ///     An error object indicating if the operation succeeded,
+        ///     and if not, what error occurred.
+        //------------------------------------------------------------------
+        virtual Error
+        LaunchNativeProcess (
+            ProcessLaunchInfo &launch_info,
+            lldb_private::NativeProcessProtocol::NativeDelegate &native_delegate,
+            NativeProcessProtocolSP &process_sp);
+
+        //------------------------------------------------------------------
+        /// Attach to an existing process on the given platform.
+        ///
+        /// This method differs from Attach() in that it returns a
+        /// NativeProcessProtocol.  Currently this is used by lldb-gdbserver.
+        ///
+        /// @param[in] pid
+        ///     pid of the process locatable by the platform.
+        ///
+        /// @param[in] native_delegate
+        ///     The delegate that will receive messages regarding the
+        ///     inferior.  Must outlive the NativeProcessProtocol
+        ///     instance.
+        ///
+        /// @param[out] process_sp
+        ///     On successful return from the method, this parameter
+        ///     contains the shared pointer to the
+        ///     NativeProcessProtocol that can be used to manipulate
+        ///     the native process.
+        ///
+        /// @return
+        ///     An error object indicating if the operation succeeded,
+        ///     and if not, what error occurred.
+        //------------------------------------------------------------------
+        virtual Error
+        AttachNativeProcess (lldb::pid_t pid,
+                             lldb_private::NativeProcessProtocol::NativeDelegate &native_delegate,
+                             NativeProcessProtocolSP &process_sp);
+
     protected:
         bool m_is_host;
         // Set to true when we are able to actually set the OS version while 
@@ -806,6 +933,7 @@ namespace lldb_private {
         bool m_system_arch_set_while_connected;
         ConstString m_sdk_sysroot; // the root location of where the SDK files are all located
         ConstString m_sdk_build;
+        ConstString m_working_dir; // The working directory which is used when installing modules that have no install path set
         std::string m_remote_url;
         std::string m_name;
         uint32_t m_major_os_version;
@@ -826,6 +954,25 @@ namespace lldb_private {
         std::string m_ssh_opts;
         bool m_ignores_remote_hostname;
         std::string m_local_cache_directory;
+        std::vector<ConstString> m_trap_handlers;
+        bool m_calculated_trap_handlers;
+        Mutex m_trap_handler_mutex;
+
+        //------------------------------------------------------------------
+        /// Ask the Platform subclass to fill in the list of trap handler names
+        ///
+        /// For most Unix user process environments, this will be a single
+        /// function name, _sigtramp.  More specialized environments may have
+        /// additional handler names.  The unwinder code needs to know when a
+        /// trap handler is on the stack because the unwind rules for the frame
+        /// that caused the trap are different.
+        ///
+        /// The base class Platform ivar m_trap_handlers should be updated by
+        /// the Platform subclass when this method is called.  If there are no
+        /// predefined trap handlers, this method may be a no-op.
+        //------------------------------------------------------------------
+        virtual void
+        CalculateTrapHandlerSymbolNames () = 0;
 
         const char *
         GetCachedUserName (uint32_t uid)
@@ -1074,7 +1221,9 @@ namespace lldb_private {
         
         bool m_ssh;
         std::string m_ssh_opts;
+
     private:
+
         DISALLOW_COPY_AND_ASSIGN(OptionGroupPlatformSSH);
     };
     
